use std::collections::HashSet;

use super::common::{Contraption, Direction};

pub fn part_one(input_lines: Vec<String>) -> String {

    let mut contraption: Vec<Vec<Contraption>> = Vec::new();
    for line in input_lines {
        let mut row: Vec<Contraption> = Vec::new();
        for c in line.chars() {
            match c {
                '.' => row.push(Contraption::Empty),
                '\\' => row.push(Contraption::MirrorLeft),
                '/' => row.push(Contraption::MirrorRight),
                '|' => row.push(Contraption::SplitterVert),
                '-' => row.push(Contraption::SplitterHoriz),
                _ => panic!("Unknown character: {}", c),
            }
        }
        contraption.push(row);
    }

    // print_contraption(&contraption);
    let mut energized: Vec<Vec<bool>> = vec![vec![false; contraption[0].len()]; contraption.len()];
    let mut memory: HashSet<(i32, i32, Direction)> = HashSet::new();
    let mut stack: Vec<(i32, i32, Direction)> = Vec::new();
    stack.push((0,0, Direction::Right));

    while let Some((i, j, direction)) = stack.pop() {

        if i >= contraption[0].len() as i32 || j >= contraption.len() as i32 || i < 0 || j < 0 {
            continue;
        }

        if memory.contains(&(i, j, direction))  {
            continue;
        } else {
            memory.insert((i, j, direction));
        }

        energized[i as usize][j as usize] = true;

        let tile = &contraption[i as usize][j as usize];
        // println!("{} {}: {:?} - {:?}", i, j, direction, tile);

        match (tile, direction) {
            (Contraption::Empty, Direction::Right) => {
                stack.push((i, j+1, Direction::Right));
            },
            (Contraption::Empty, Direction::Left) => {
                stack.push((i, j-1, Direction::Left));
            },
            (Contraption::Empty, Direction::Up) => {
                stack.push((i-1, j, Direction::Up));
            },
            (Contraption::Empty, Direction::Down) => {
                stack.push((i+1, j, Direction::Down));
            },
            (Contraption::MirrorLeft, Direction::Right) => {
                stack.push((i+1, j, Direction::Down));
            }
            (Contraption::MirrorLeft, Direction::Left) => {
                stack.push((i-1, j, Direction::Up));
            },
            (Contraption::MirrorLeft, Direction::Up) => {
                stack.push((i, j-1, Direction::Left));
            },
            (Contraption::MirrorLeft, Direction::Down) => {
                stack.push((i, j+1, Direction::Right));
            },
            (Contraption::MirrorRight, Direction::Right) => {
                stack.push((i-1, j, Direction::Up));
            },
            (Contraption::MirrorRight, Direction::Left) => {
                stack.push((i+1, j, Direction::Down));
            },
            (Contraption::MirrorRight, Direction::Up) => {
                stack.push((i, j+1, Direction::Right));
            },
            (Contraption::MirrorRight, Direction::Down) => {
                stack.push((i, j-1, Direction::Left));
            },
            (Contraption::SplitterVert, Direction::Right | Direction::Left) => {
                stack.push((i-1, j, Direction::Up));
                stack.push((i+1, j, Direction::Down));
            },
            (Contraption::SplitterVert, Direction::Down) => {
                stack.push((i+1, j, Direction::Down));
            },
            (Contraption::SplitterVert, Direction::Up) => {
                stack.push((i-1, j, Direction::Up));
            },
            (Contraption::SplitterHoriz, Direction::Up | Direction::Down) => {
                stack.push((i, j-1, Direction::Left));
                stack.push((i, j+1, Direction::Right));
            },
            (Contraption::SplitterHoriz, Direction::Left) => {
                stack.push((i, j-1, Direction::Left));
            },
            (Contraption::SplitterHoriz, Direction::Right) => {
                stack.push((i, j+1, Direction::Right));
            }
        }
    }

    // print_energized(&energized);

    let energized_count = energized.iter().flatten().filter(|x| **x).count();
    energized_count.to_string()
}